package Tietorakenteet;

import java.util.ArrayList;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Rekisteri {
    private  final Jasenet jasenet = new Jasenet();
    private final Hiihdot hiihdot = new Hiihdot();
    
    
    /**
     * Muodostaja
     */
    public Rekisteri(){
    }
    
    
    /**
     * Lisää jäsenen taulukkoon.
     * @param jasen lisättävän jäsenen viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * jasenet.getLkm() === 2;
     * </pre>
     */
    public void lisaa(Jasen jasen){
        jasenet.lisaa(jasen);
    }
    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(eka);
     * jasenet.lisaa(toka);
     * rekisteri.annaJasen(0) === eka;
     * </pre>
     */
    public Jasen annaJasen(int i) {
        return jasenet.anna(i);
    }
    
    
    /**
     * Lisää hiihto listaan.
     * @param hiihto lisättävän hiihdon viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.getHiihdot() === 2;
     * </pre>
     */
    public void lisaaHiihto(Hiihto hiihto){
        hiihdot.lisaa(hiihto);
    }
    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.annaHiihto(0) === eka;
     * </pre>
     */
    public Hiihto annaHiihto(int i) {
        return hiihdot.anna(i);
    }
    
    
    /**
     * @param id hiihtajan ID
     * @return halutun jäsenen kaikki hiihdot
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * rekisteri.getJasenenHiihdot(1) === "[eka, toka]";
     * rekisteri.getJasenenHiihdot(2) === "[kolmas]";
     * </pre>
    */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        return hiihdot.getJasenenHiihdot(id);
    }
    
    
    /**
     * Lukee jäsenistön tiedostosta. TODO lukeminen tiedostosta
     * @param nimi jota käytetään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        jasenet.lueTiedostosta(nimi);
    }
    
    
    /**
     * Poistaa jäsenet ja hiihdot, joilla on annettu nro
     * TODO jäsenien ja hiihtojen poisto ko. luokista. 
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void talleta() throws SailoException{
        jasenet.talleta();
    }
    
    
    /**
     * @return jäsenistön lkm
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * </pre>
     */
    public int getLkm() {
        return jasenet.getLkm();
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten lukumäärä
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * rekisteri.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.getHiihdot();
    }
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Rekisteri rekisteri = new Rekisteri();
        
        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen nokkaraka = new Jasen(), mietaa = new Jasen();
            

            rekisteri.lisaa(nokkaraka);
            rekisteri.lisaa(mietaa);

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < rekisteri.getLkm(); i++) {
                Jasen jasen = rekisteri.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                System.out.println();
                System.out.println();
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }
}